#!/bin/bash

# This script traces information from an EDG id or from 
# an JM-Contact string, from an RB perspective

TMP=`mktemp -q`
TIME=`date`
CONTACT="Romain.Wartel@cern.ch"

#EDG_ID=$1
CALL=$2

LOG_DIR=`/bin/grep -v \# /etc/lcg-logs.conf | /bin/grep log_dir | sed -u -e 's/.*=//g'| sed -u -e 's/\s//g'`
MYSQL_U=`/bin/grep -v \# /etc/lcg-logs.conf | /bin/grep mysql_user | sed -u -e 's/.*=//g'| sed -u -e 's/\s//g'`
MYSQL_P=`/bin/grep -v \# /etc/lcg-logs.conf | /bin/grep mysql_password | sed -u -e 's/.*=//g'| sed -u -e 's/\s//g'`
MYSQL_D=`/bin/grep -v \# /etc/lcg-logs.conf | /bin/grep mysql_db | sed -u -e 's/.*=//g'| sed -u -e 's/\s//g'`




if [ ${CALL} == "EDGid" ]; then
	EDG_ID=$1
	CONDOR_REF="Input parameters"
        echo "####################################################################"
	echo "${TIME}: Tracing EDG id: ${EDG_ID}"
        echo "####################################################################"

else

	JM=$1
        echo "####################################################################"
	echo "Tracing JM-Contact: ${JM}"
        echo "####################################################################"

	JM1=`echo ${JM}|awk -F ":" '{print $3}'|awk -F "/" '{print $3}'`
	JM2=`echo ${JM}|awk -F ":" '{print $3}'|awk -F "/" '{print $2}'`


# Discovering EDG_ID from JM-Contact string

# Typically, we have this:
# user.log:Dec 13 06:44:37 lxb2016 glite-/var/edgwl/logmonitor/log/events.log: 13 Dec, 06:44:37 -C- CondorMonitor::processEvent(...): Contacts lxb2018.cern.ch:2119/jobmanager-lcgpbs, https://lxb2018.cern.ch:20006/30103/1134452664/.
# user.log:Dec 13 06:44:37 lxb2016 glite-/var/edgwl/logmonitor/log/events.log: 13 Dec, 06:44:37 -C- CondorMonitor::processEvent(...): EDG id = https://lxb2016.cern.ch:9000/3K2UeOQ0uim78JN0Xi3DtA

	CondorEvents=`grep " Contacts " ${LOG_DIR}* |grep "CondorMonitor::processEvent" |grep -e "${JM2}\/${JM1}" | sed -e 's/Contacts .*//g' > ${TMP}`
	CONDOR_REF=`cat ${TMP}  |awk -F ":" '{print $2 ":" $3 ":" $4 }'|sort -u`
	CondorEvents=`cat ${TMP} |awk -F ":" '{print $2 ":" $3 ":" $4 ":" $5 ":" $6 ":" $7 ":" $8 ":" $9 }'|sort -u > ${TMP}`

	EDG_ID=`cat ${TMP} |awk -F "NOSEP" -v LOG_DIR=${LOG_DIR} '{print "grep -e \"" $1 "\" " LOG_DIR "*" }'|bash |grep "EDG id = " |sed -e 's/.*EDG id = \(\S*\)/\1/g' |sort -u`

fi

if [ "x${EDG_ID}" == "x" ]; then
	echo "--"
	echo "Unable to retrieve EDG_ID from RB"
	echo "--"
else

	echo "--"
	echo "Source:"
	echo  "${CONDOR_REF}"
	echo "* EDG_ID = ${EDG_ID}"

	ID_SUFFIX=`echo ${EDG_ID} |awk -F "/" '{print $4}'`

	RB=`echo ${EDG_ID} |awk -F "/" '{print $3}'|awk -F ":" '{print $1}'`
	RB_SHORT=`echo ${RB}|awk -F "." '{print $1}'`

	echo "* RB = ${RB}"


# Looking for Network server information

	NETSERVER=`grep -e "${ID_SUFFIX}" ${LOG_DIR}* |grep "Executable = " > ${TMP}`
	NETSERVER_REF=`cat ${TMP} |awk -F":" '{print $2 ":" $3 ":" $4}'|sort -u`
	EXEC=`cat ${TMP} | sed -e 's/.*Executable = "\(\S*\)"; .*/\1/'|sort -u`
	DN=`cat ${TMP} | sed -e 's/.*CertificateSubject = "\(.*\)"; .*/\1/' | sed -e 's/";.*//g' | sort -u`
	JOB_REQUIREMENT=`cat ${TMP} | sed -e 's/.*requirements = \(.*\)) /\1/'|sed -e 's/;.*//g' |sort -u`
	MYPROXY=`cat ${TMP} | sed -e 's/.*MyProxyServer = "\(.*\)";/\1/'|sed -e 's/";.*//g'|sort -u`

	ARG=`cat ${TMP} |grep "Arguments = "`
	if [ "x$ARG" == "x" ]; then
		ARGS="(no arguments)"
	else
		ARGS=`cat ${TMP} | sed -e 's/.*Arguments = "\(.*\)";/\1/'|sed -e 's/";.*//g'|sort -u`
	fi

	TIME=`cat ${TMP} | sed -e 's/\(.*\)${RB_SHORT}.*/\1/g'|awk -F ":" '{print $2 ":" $3 ":" $4 }'|awk '{print $1 " "  $2 " " $3}' |sort -u`

	if [ "x${NETSERVER_REF}" != "x" ]; then

		echo "--"
		echo "Source:"
		echo  "${NETSERVER_REF}"
		echo "* Executable = ${EXEC}"
		echo "* Job arguments = ${ARGS}"
		echo "* Job Requirement = ${JOB_REQUIREMENT}"
		echo "* Submission time = ${TIME}"
		echo "* DN = ${DN}"
		echo "* MyProxy Server = ${MYPROXY}"

	fi

# Information from Condor and Job log

	DESTINATION=`grep -e "${ID_SUFFIX}" ${LOG_DIR}* |grep "DESTINATION=" > ${TMP}`
	CE=`cat ${TMP} |sed -e 's/.*DESTINATION=\(\S*\) .*/\1/g'| sed '/^$/d' | sort -u`
	CE_SHORT=`cat ${TMP} |sed -e 's/.*DESTINATION=\(\S*\) .*/\1/g'| sed '/^$/d' | sort -u|awk -F":" {'print $1'} |awk -F"." {'print $1'}`
	CE_REF=`cat ${TMP} |awk -F":" '{print $2 ":" $3 ":" $4}'|sort -u`
	CONDORID=`cat ${TMP} |sed -e 's/.*CONDORID=\(\S*\) .*/\1/g'| sed '/^$/d' | sort -u`

	if [ "x${CE_REF}" != "x" ]; then

		echo "--"
		echo "Source:"
		echo  "${CE_REF}"
		echo "* CE = ${CE}"
		echo "* CondorID = ${CONDORID}"
	
	fi

# FTP Interactions
	ID_SUFFIX2=`echo ${ID_SUFFIX} |sed -e 's/_/_5f/g'`
	FTP=`grep -e "${ID_SUFFIX2}" ${LOG_DIR}* |grep " PROG=wuftpd "  > ${TMP}`
	DEST_IP=`cat ${TMP} | sed -e 's/.*DEST=1\[\(\S*\)\] .*/\1/g'|sort -u`
	RB_UID=`cat ${TMP} | sed -e 's/.*USER=\(\S*\) .*/\1/g'|sort -u` 
	FTP_REF=`cat ${TMP} |awk -F":" '{print $2 ":" $3 ":" $4}'|sort -u`

	if [ "x${FTP_REF}" != "x" ]; then

		echo "--"
		echo "Source:"
		echo "${FTP_REF}"
		echo "* RB_UID = ${RB_UID}"
		echo "* FTP Destinations:"
		echo "${DEST_IP}" 
	fi

# Now trying to get the JM-CONTACT *without* the CondorG files

# Typically, we have this:
# user.log:Dec 13 06:44:37 lxb2016 glite-/var/edgwl/logmonitor/log/events.log: 13 Dec, 06:44:37 -C- CondorMonitor::processEvent(...): Contacts lxb2018.cern.ch:2119/jobmanager-lcgpbs, https://lxb2018.cern.ch:20006/30103/1134452664/.
# user.log:Dec 13 06:44:37 lxb2016 glite-/var/edgwl/logmonitor/log/events.log: 13 Dec, 06:44:37 -C- CondorMonitor::processEvent(...): EDG id = https://lxb2016.cern.ch:9000/3K2UeOQ0uim78JN0Xi3DtA

	if [ "x${JM}" == "x" ]; then

		CondorEvents=`grep -e "${ID_SUFFIX}" ${LOG_DIR}* |grep "CondorMonitor::processEvent" |sed -e 's/EDG id.*//g'|sed -e 's/.*${RB_SHORT}//g' > ${TMP}`
		CONDOR_REF=`cat ${TMP} |awk -F":" '{print $2 ":" $3 ":" $4}'|sort -u`
		CondorEvents=`cat ${TMP} |awk -F ":" '{print $5 ":" $6 ":" $7}'|sort -u > ${TMP}`
		JM=`cat ${TMP} |awk -F "NOSEP" -v LOG_DIR=${LOG_DIR} '{print "grep -e \"" $1 "\" " LOG_DIR "*" }'|bash |grep "Contacts"|sed -e 's/.*, \(\S*\)\./\1/g'|sort -u`

	fi
	if [ "x${JM}" == "x" ]; then

		echo "No JM Contact string information available."
	else
		if [ "x${CONDOR_REF}" != "x" ]; then
			echo "--"
			echo "Source:"
			echo  "${CONDOR_REF}"
			echo "* JM-Contact = ${JM}"
			JM1=`echo ${JM}|awk -F ":" '{print $3}'|awk -F "/" '{print $2}'`
			JM2=`echo ${JM}|awk -F ":" '{print $3}'|awk -F "/" '{print $3}'`

		fi

	fi

rm -f ${TMP}

fi




if [ "x${JM1}" == "x" ]; then
#	echo "JM1 is empty"| mail -s "Problem with EDG id: ${EDG_ID}" ${CONTACT}
	exit
fi
if [ "x${JM2}" == "x" ]; then
#        echo "JM2 is empty"| mail -s "Problem with EDG id: ${EDG_ID}" ${CONTACT}
	exit
fi


# Checking if a similar entry already exists

        SQL="select count(*) from job where JM1='${JM1}' and JM2='${JM2}' and CE='${CE_SHORT}';"
        RES=`mysql -u ${MYSQL_U} -p${MYSQL_P} ${MYSQL_D} -B -e "${SQL}" |grep -v "count"`

        if [ "$RES" != "0" ]; then
                # Updating similar entry
                echo "Existing entry: updating database..."
		SQL="update job set JM1='${JM1}'"
                if [ "x${EDG_ID}" != "x" ]; then SQL=${SQL}",EDGid='${EDG_ID}'";fi
                if [ "x${RB}" != "x" ]; then SQL=${SQL}",RB='${RB}'";fi
                if [ "x${EXEC}" != "x" ]; then SQL=${SQL}",Executable='${EXEC}'";fi
                if [ "x${ARGS}" != "x" ]; then SQL=${SQL}",Job_Args='${ARGS}'";fi
                if [ "x${JOB_REQUIREMENT}" != "x" ]; then SQL=${SQL}",Job_Req='${JOB_REQUIREMENT}'";fi
                if [ "x${TIME}" != "x" ]; then SQL=${SQL}",Submission_Time='${TIME}'";fi
                if [ "x${RB_UID}" != "x" ]; then SQL=${SQL}",RB_UID='${RB_UID}'";fi
                if [ "x${DN}" != "x" ]; then SQL=${SQL}",RB_DN='${DN}'";fi
                if [ "x${MYPROXY}" != "x" ]; then SQL=${SQL}",My_Proxy='${MYPROXY}'";fi
                if [ "x${CE}" != "x" ]; then SQL=${SQL}",Target_CE='${CE}'";fi
		if [ "x${CE_SHORT}" != "x" ]; then SQL=${SQL}",CE='${CE_SHORT}'";fi
                if [ "x${CONDORID}" != "x" ]; then SQL=${SQL}",CondorID='${CONDORID}'";fi
                if [ "x${DEST_IP}" != "x" ]; then SQL=${SQL}",FTP_Dest='${DEST_IP}'";fi
                SQL=${SQL}" where JM1='${JM1}' and JM2='${JM2}' and CE='${CE_SHORT}';"
		RES=`mysql -u ${MYSQL_U} -p${MYSQL_P} ${MYSQL_D} -B -e "${SQL}"`
                echo ${RES}
        else
                echo "New entry: adding to database..."
                SQL="insert into job set JM1='${JM1}',JM2='${JM2}',EDGid='${EDG_ID}',RB='${RB}',Executable='${EXEC}',Job_Args='${ARGS}',Job_Req='${JOB_REQUIREMENT}',Submission_Time='${TIME}',RB_UID='${RB_UID}',RB_DN='${DN}',My_Proxy='${MYPROXY}',Target_CE='${CE}',CondorID='${CONDORID}',FTP_Dest='${DEST_IP}'"
		echo $SQL

                RES=`mysql -u ${MYSQL_U} -p${MYSQL_P} ${MYSQL_D} -B -e "${SQL}"`
                echo ${RES}
        fi



# We now pass the JM-Contact string to the CE module

#if [ "$CALL" != "scan_CE" ]; then
#	if [ "x${JM}" != "x" ]; then
#
#		echo "--"
#		echo "Retrieving information from CE"
#		echo "--"
#		/usr/bin/scan_CE.sh ${JM} scan_RB
#
#	fi
#fi
